@@ -568,9 +568,6 @@ public abstract class AbstractIoFuture<T> implements IoFuture<T> {
         do {
             oldState = getState();
             newState = oldState.withNotifier(getNotifierExecutor(), this, notifier, attachment);
-            if (oldState == newState) {
-                return this;
-            }
         } while (! compareAndSetState(oldState, newState));
         return this;
     }
@@ -722,7 +719,7 @@ public abstract class AbstractIoFuture<T> implements IoFuture<T> {
             try {
                 notifier.notify(future, attachment);
             } catch (Throwable t) {
-                futureMsg.notifierFailed(t, notifier);
+                futureMsg.notifierFailed(t, notifier, attachment);
             }
         }
     }
