@@ -65,6 +65,39 @@ public final class Channels {
         }
     }
 
+    /**
+     * Simple utility method to execute a blocking flush on a writable channel. The method blocks until there are no
+     * remaining bytes in the send queue or the timeout is reached.
+     *
+     * @param channel the writable channel
+     * @param time the amount of time to wait
+     * @param unit the unit of time to wait
+     * @return true if the channel was successfully flushed, false if the timeout was reached
+     * @throws IOException if an I/O exception occurs
+     *
+     * @since 3.8
+     */
+    public static boolean flushBlocking(SuspendableWriteChannel channel, long time, TimeUnit unit) throws IOException {
+        // In the fast path, the timeout is not used because bytes can be flushed without blocking.
+        if (channel.flush()) {
+            return true;
+        }
+        long remaining = unit.toNanos(time);
+        long now = System.nanoTime();
+        do {
+            // awaitWritable may return spuriously so looping is required.
+            channel.awaitWritable(remaining, TimeUnit.NANOSECONDS);
+            // flush prior to recalculating remaining time to avoid a nanoTime
+            // invocation in the optimal path.
+            if (channel.flush()) {
+                return true;
+            }
+            // Nanotime must only be used in comparison with another nanotime value
+            // This implementation allows us to avoid immediate subsequent nanoTime calls
+        } while ((remaining -= Math.max(-now + (now = System.nanoTime()), 0L)) > 0L);
+        return false;
+    }
+
     /**
      * Simple utility method to execute a blocking write shutdown on a writable channel.  The method blocks until the
      * channel's output side is fully shut down.
@@ -79,6 +112,23 @@ public final class Channels {
         flushBlocking(channel);
     }
 
+    /**
+     * Simple utility method to execute a blocking write shutdown on a writable channel.  The method blocks until the
+     * channel's output side is fully shut down or the timeout is reached.
+     *
+     * @param channel the writable channel
+     * @param time the amount of time to wait
+     * @param unit the unit of time to wait
+     * @return true if the channel was successfully flushed, false if the timeout was reached
+     * @throws IOException if an I/O exception occurs
+     *
+     * @since 3.8
+     */
+    public static boolean shutdownWritesBlocking(SuspendableWriteChannel channel, long time, TimeUnit unit) throws IOException {
+        channel.shutdownWrites();
+        return flushBlocking(channel, time, unit);
+    }
+
     /**
      * Simple utility method to execute a blocking write on a byte channel.  The method blocks until the bytes in the
      * buffer have been fully written.  To ensure that the data is sent, the {@link #flushBlocking(SuspendableWriteChannel)}
@@ -311,13 +361,27 @@ public final class Channels {
      * @since 1.2
      */
     public static <C extends ReadableByteChannel & SuspendableReadChannel> int readBlocking(C channel, ByteBuffer buffer, long time, TimeUnit unit) throws IOException {
+        // In the fast path, the timeout is not used because bytes can be read without blocking.
         int res = channel.read(buffer);
-        if (res == 0 && buffer.hasRemaining()) {
-            channel.awaitReadable(time, unit);
-            return channel.read(buffer);
-        } else {
+        if (res != 0) {
             return res;
         }
+        long remaining = unit.toNanos(time);
+        long now = System.nanoTime();
+        while (buffer.hasRemaining() && remaining > 0) {
+            // awaitReadable may return spuriously, so looping is required.
+            channel.awaitReadable(remaining, TimeUnit.NANOSECONDS);
+            // read prior to recalculating remaining time to avoid a nanoTime
+            // invocation in the optimal path.
+            res = channel.read(buffer);
+            if (res != 0) {
+                return res;
+            }
+            // Nanotime must only be used in comparison with another nanotime value
+            // This implementation allows us to avoid immediate subsequent nanoTime calls
+            remaining -= Math.max(-now + (now = System.nanoTime()), 0L);
+        }
+        return res;
     }
 
     /**
@@ -357,13 +421,28 @@ public final class Channels {
      * @since 1.2
      */
     public static <C extends ScatteringByteChannel & SuspendableReadChannel> long readBlocking(C channel, ByteBuffer[] buffers, int offs, int len, long time, TimeUnit unit) throws IOException {
+        // In the fast path, the timeout is not used because bytes can be read
+        // without blocking or interaction with the precise clock.
         long res = channel.read(buffers, offs, len);
-        if (res == 0L && Buffers.hasRemaining(buffers, offs, len)) {
-            channel.awaitReadable(time, unit);
-            return channel.read(buffers, offs, len);
-        } else {
+        if (res != 0L) {
             return res;
         }
+        long remaining = unit.toNanos(time);
+        long now = System.nanoTime();
+        while (Buffers.hasRemaining(buffers, offs, len) && remaining > 0) {
+            // awaitReadable may return spuriously, looping is required.
+            channel.awaitReadable(remaining, TimeUnit.NANOSECONDS);
+            // read prior to recalculating remaining time to avoid a nanoTime
+            // invocation in the optimal path.
+            res = channel.read(buffers, offs, len);
+            if (res != 0) {
+                return res;
+            }
+            // Nanotime must only be used in comparison with another nanotime value
+            // This implementation allows us to avoid immediate subsequent nanoTime calls
+            remaining -= Math.max(-now + (now = System.nanoTime()), 0L);
+        }
+        return res;
     }
 
     /**
@@ -399,13 +478,27 @@ public final class Channels {
      * @since 1.2
      */
     public static <C extends ReadableMessageChannel> int receiveBlocking(C channel, ByteBuffer buffer, long time, TimeUnit unit) throws IOException {
+        // In the fast path, the timeout is not used because bytes can be read without blocking.
         int res = channel.receive(buffer);
-        if ((res) == 0) {
-            channel.awaitReadable(time, unit);
-            return channel.receive(buffer);
-        } else {
+        if (res != 0) {
             return res;
         }
+        long remaining = unit.toNanos(time);
+        long now = System.nanoTime();
+        while (buffer.hasRemaining() && remaining > 0) {
+            // awaitReadable may return spuriously, looping is required.
+            channel.awaitReadable(remaining, TimeUnit.NANOSECONDS);
+            // read prior to recalculating remaining time to avoid a nanoTime
+            // invocation in the optimal path.
+            res = channel.receive(buffer);
+            if (res != 0) {
+                return res;
+            }
+            // Nanotime must only be used in comparison with another nanotime value
+            // This implementation allows us to avoid immediate subsequent nanoTime calls
+            remaining -= Math.max(-now + (now = System.nanoTime()), 0L);
+        }
+        return res;
     }
 
     /**
@@ -445,13 +538,25 @@ public final class Channels {
      * @since 1.2
      */
     public static <C extends ReadableMessageChannel> long receiveBlocking(C channel, ByteBuffer[] buffers, int offs, int len, long time, TimeUnit unit) throws IOException {
+        // In the fast path, the timeout is not used because bytes can be read without blocking.
         long res = channel.receive(buffers, offs, len);
-        if ((res) == 0) {
-            channel.awaitReadable(time, unit);
-            return channel.receive(buffers, offs, len);
-        } else {
+        if (res != 0L) {
             return res;
         }
+        long remaining = unit.toNanos(time);
+        long now = System.nanoTime();
+        while (Buffers.hasRemaining(buffers, offs, len) && remaining > 0) {
+            // awaitReadable may return spuriously, looping is required.
+            channel.awaitReadable(remaining, TimeUnit.NANOSECONDS);
+            res = channel.receive(buffers, offs, len);
+            if (res != 0) {
+                return res;
+            }
+            // Nanotime must only be used in comparison with another nanotime value
+            // This implementation allows us to avoid immediate subsequent nanoTime calls
+            remaining -= Math.max(-now + (now = System.nanoTime()), 0L);
+        }
+        return res;
     }
 
     /**
