@@ -19,7 +19,6 @@
 
 package org.xnio.ssl;
 
-import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
@@ -28,7 +27,6 @@ import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertTrue;
 
 import java.io.IOException;
-import java.lang.reflect.Field;
 import java.net.Inet4Address;
 import java.net.InetSocketAddress;
 import java.net.URL;
@@ -36,14 +34,13 @@ import java.nio.channels.Channel;
 import java.util.concurrent.CancellationException;
 
 import javax.net.ssl.SSLContext;
-import javax.net.ssl.SSLEngine;
 
+import org.junit.After;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Ignore;
 import org.junit.Test;
 import org.xnio.ChannelListener;
-import org.xnio.ChannelListeners;
 import org.xnio.IoFuture;
 import org.xnio.OptionMap;
 import org.xnio.Options;
@@ -54,6 +51,7 @@ import org.xnio.channels.AcceptingChannel;
 import org.xnio.channels.BoundChannel;
 import org.xnio.channels.ConnectedSslStreamChannel;
 import org.xnio.channels.ConnectedStreamChannel;
+import org.xnio.mock.XnioIoThreadMock;
 import org.xnio.mock.XnioWorkerMock;
 
 /**
@@ -63,9 +61,6 @@ import org.xnio.mock.XnioWorkerMock;
  */
 @SuppressWarnings("deprecation")
 public class JsseXnioSslTestCase {
-    private static final Field setterField;
-    private static final Field connectionEngineField;
-    private static final Field conduitEngineField;
     private static final String KEY_STORE_PROPERTY = "javax.net.ssl.keyStore";
     private static final String KEY_STORE_PASSWORD_PROPERTY = "javax.net.ssl.keyStorePassword";
     private static final String TRUST_STORE_PROPERTY = "javax.net.ssl.trustStore";
@@ -74,24 +69,6 @@ public class JsseXnioSslTestCase {
     private static final String DEFAULT_KEY_STORE_PASSWORD = "apiTest";
 
     private static final int SERVER_PORT = 23456;
-    
-    static {
-        try {
-            Class<?> delegateSetterClass = JsseXnioSslTestCase.class.getClassLoader().loadClass(ChannelListeners.class.getName()+ "$DelegatingSetter");
-            setterField = delegateSetterClass.getDeclaredField("setter");
-            setterField.setAccessible(true);
-            connectionEngineField = JsseSslStreamConnection.class.getDeclaredField("sslConduitEngine");
-            connectionEngineField.setAccessible(true);
-            conduitEngineField = JsseSslConduitEngine.class.getDeclaredField("engine");
-            conduitEngineField.setAccessible(true);
-        } catch (ClassNotFoundException e) {
-            throw new RuntimeException(e);
-        } catch (NoSuchFieldException e) {
-            throw new RuntimeException(e);
-        } catch (SecurityException e) {
-            throw new RuntimeException(e);
-        }
-    }
 
     @BeforeClass
     public static void setKeyStoreAndTrustStore() {
@@ -114,14 +91,23 @@ public class JsseXnioSslTestCase {
     private Xnio xnio;
     private XnioWorkerMock worker;
     private XnioSsl xnioSsl;
+    private XnioIoThreadMock threadMock;
 
     @Before
     public void init() throws Exception {
         serverAddress = new InetSocketAddress(Inet4Address.getByAddress(new byte[] { 127, 0, 0, 1 }), SERVER_PORT);
         xnio = Xnio.getInstance("xnio-mock", JsseXnioSslTestCase.class.getClassLoader());
         worker = (XnioWorkerMock) xnio.createWorker(OptionMap.EMPTY);
+        threadMock = worker.chooseThread();
+        threadMock.start();
         xnioSsl = xnio.getSslProvider(OptionMap.EMPTY);
     }
+
+    @After
+    public void closeIoThread() {
+        threadMock.closeIoThread();
+    }
+
     @Test
     public void testCreateSslConnectionServer() throws Exception {
         final TestChannelListener<AcceptingChannel<SslConnection>> openListener = new TestChannelListener<AcceptingChannel<SslConnection>>();
@@ -256,7 +242,6 @@ public class JsseXnioSslTestCase {
         final InetSocketAddress localAddress = new InetSocketAddress(500);
         final OptionMap.Builder builder = OptionMap.builder();
         builder.set(Options.SSL_PROTOCOL, "TLSv1");
-        builder.set(Options.SSL_PROVIDER, "SunJSSE");
         builder.set(Options.SSL_CLIENT_SESSION_CACHE_SIZE, 20000);
         builder.set(Options.SSL_CLIENT_SESSION_TIMEOUT, 80);
         builder.set(Options.SSL_SERVER_SESSION_CACHE_SIZE, 30000);
@@ -272,8 +257,6 @@ public class JsseXnioSslTestCase {
         assertEquals(localAddress, connection.getLocalAddress());
         assertEquals(serverAddress, connection.getPeerAddress());
         assertTrue(connection.getOption(Options.TCP_NODELAY));
-        final SSLEngine engine = (SSLEngine) conduitEngineField.get((JsseSslConduitEngine) connectionEngineField.get(connection));
-        assertTrue(engine.getEnableSessionCreation());
     }
 
     @Test
@@ -296,6 +279,7 @@ public class JsseXnioSslTestCase {
         optionMapBuilder.set(Options.MULTICAST, true);
         optionMapBuilder.set(Options.READ_TIMEOUT, 1000);
         optionMapBuilder.set(Options.SSL_ENABLED_PROTOCOLS, Sequence.of(enabledProtocols));
+        optionMapBuilder.set(Options.SSL_STARTTLS, true);
         final IoFuture<? extends StreamConnection> ioFuture = xnioSsl.openSslConnection(worker, localAddress,
                 serverAddress, openListener, bindListener, optionMapBuilder.getMap());
         final StreamConnection connection = ioFuture.get();
@@ -312,11 +296,6 @@ public class JsseXnioSslTestCase {
         assertTrue(boundChannel.getOption(Options.MULTICAST));
         assertEquals(1000, (int) connection.getOption(Options.READ_TIMEOUT));
         assertEquals(1000, (int) boundChannel.getOption(Options.READ_TIMEOUT));
-
-        final SSLEngine engine = (SSLEngine) conduitEngineField.get((JsseSslConduitEngine) connectionEngineField.get(connection));
-        assertFalse(engine.getEnableSessionCreation());
-        assertArrayEquals(validCipherSuites, engine.getEnabledCipherSuites());
-        assertArrayEquals(validProtocols, engine.getEnabledProtocols());
     }
 
     @Test
@@ -369,7 +348,6 @@ public class JsseXnioSslTestCase {
         final InetSocketAddress localAddress = new InetSocketAddress(500);
         final OptionMap.Builder builder = OptionMap.builder();
         builder.set(Options.SSL_PROTOCOL, "TLSv1");
-        builder.set(Options.SSL_PROVIDER, "SunJSSE");
         builder.set(Options.SSL_CLIENT_SESSION_CACHE_SIZE, 20000);
         builder.set(Options.SSL_CLIENT_SESSION_TIMEOUT, 80);
         builder.set(Options.SSL_SERVER_SESSION_CACHE_SIZE, 30000);
@@ -406,6 +384,7 @@ public class JsseXnioSslTestCase {
         optionMapBuilder.set(Options.MULTICAST, true);
         optionMapBuilder.set(Options.READ_TIMEOUT, 1000);
         optionMapBuilder.set(Options.SSL_ENABLED_PROTOCOLS, Sequence.of(enabledProtocols));
+        optionMapBuilder.set(Options.SSL_STARTTLS, true);
         final IoFuture<? extends ConnectedStreamChannel> ioFuture = xnioSsl.connectSsl(worker, localAddress,
                 serverAddress, openListener, bindListener, optionMapBuilder.getMap());
         final ConnectedStreamChannel channel = ioFuture.get();
